#!/usr/bin/env bash

set -o errexit # Exit immediately if a simple command (see SHELL GRAMMAR above) exits with a non-zero status.

function usage {
echo "Usage: `basename $0` url"
exit -1
}

chksumfile=CHECKSUM.MD5
if [ "`uname -s`" = "Linux" ]
then
	MD5=md5sum
fi

if [ "`uname -s`" = "FreeBSD" ] || [ "`uname -s`" = "Darwin" ] 
then
        MD5=md5
fi

trap "usage" ERR
trap "rm -f \"$chksumfile\"" EXIT

url="$1"

dir=$RANDOM
mkdir $dir
cd $dir
test -z "$url" && usage

rm -f "$chksumfile"
wget -q $url/$chksumfile
for f in `cat "$chksumfile"| awk '{ print $2}' | sed -e 's/(//' -e 's/)//'`
do
        echo -n "Checking $f ... "
        rm -f "$f"
        wget -q $url/$f
        if [ "`$MD5 "$f"`" = "`cat "$chksumfile" | grep "$f"`" ]
        then
                echo "OK"
        else
                echo "Checksum mismatches"
                exit -1
        fi
        rm -f "$f"
done
cd -
rm -r $dir
